#include <MDR32F9Qx_port.h>
#include <MDR32F9Qx_rst_clk.h>
#include <MDR32F9Qx_bkp.h>
#include <MDR32F9Qx_power.h>

#include <stdio.h>

void Led_Init(void);
void Delay(int waitTicks);
void vError(void);
void RTC_Init(void);
void LSE_Pin_Init(void);
void BKP_RTC_ClearFlagStatus(uint32_t BKP_RTC_FLAG);

#define LED_PERIOD 300000
#define VD3 PORT_Pin_0
#define VD4 PORT_Pin_1
#define LED_ALL VD3 | VD4

#define ALARM_TIMEOUT 5

uint32_t i, cnt, timeout_lse;


int main(){
  /* Настройка выводов для работы со светодиодами */
  Led_Init();
  
  /* Светодиод LED0 мигает 5 раз и выключается */
  for(i = 0; i < 5; i++){
    PORT_SetBits(MDR_PORTC, VD3);  // LED0 ON
    Delay(LED_PERIOD);
    
    PORT_ResetBits(MDR_PORTC, VD3); // LED0 OFF
    Delay(LED_PERIOD);
  }
  
  RST_CLK_PCLKcmd(RST_CLK_PCLK_BKP,ENABLE);
  
  if((MDR_BKP->REG_0F & BKP_REG_0F_RTC_EN) == 0){ // Если часы RTC выключены
    RTC_Init();
  }
  
  /* Установка значения ALRM */
  BKP_RTC_WaitForUpdate();
  BKP_RTC_ClearFlagStatus(BKP_RTC_FLAG_ALRF);
  
  BKP_RTC_WaitForUpdate();
  cnt = BKP_RTC_GetCounter();
  BKP_RTC_SetAlarm(ALARM_TIMEOUT + cnt); // 5 секунд
  
  BKP_RTC_WaitForUpdate();
  BKP_RTC_ITConfig(BKP_RTC_IT_ALRF, ENABLE);
  
  /* Переход в режим STANDBY */
  MDR_BKP->REG_0F |= (1 << BKP_REG_0F_STANDBY_Pos);
  
  
  /* Код далее не выполняется */
  while(1){
    PORT_SetBits(MDR_PORTC, VD4);  // LED1 ON
    Delay(LED_PERIOD);
    
    PORT_ResetBits(MDR_PORTC, VD4); // LED1 OFF
    Delay(LED_PERIOD);
  }
}


void RTC_Init(){
  /* RTC reset */
  BKP_RTC_Reset(ENABLE);
  BKP_RTC_Reset(DISABLE);
  
  /* Configure RTC_LSE as RTC clock source */
  //LSE_Pin_Init();
  RST_CLK_LSEconfig(RST_CLK_LSE_ON);
  
  timeout_lse = 1000;
  while((RST_CLK_LSEstatus()!=SUCCESS) && (--timeout_lse)){}
  if(!timeout_lse)
    vError();
  
  BKP_RTCclkSource(BKP_RTC_LSEclk);
  
  /* Set the RTC counter value */
  BKP_RTC_WaitForUpdate();
  BKP_RTC_SetCounter(1);
  
  /* Set the RTC prescaler value */
  BKP_RTC_WaitForUpdate();
  BKP_RTC_SetPrescaler(32768); // Отсчёт секунд (LSE)
  
  /* RTC enable */
  BKP_RTC_WaitForUpdate();
  BKP_RTC_Enable(ENABLE);
}

void LSE_Pin_Init(){
  PORT_InitTypeDef LSE_Pin_InitStruct;
  
  RST_CLK_PCLKcmd (RST_CLK_PCLK_PORTE, ENABLE);
  
  PORT_StructInit(&LSE_Pin_InitStruct);
  
  LSE_Pin_InitStruct.PORT_Pin = PORT_Pin_6;
  PORT_Init(MDR_PORTE, &LSE_Pin_InitStruct);
}


void Led_Init(){
  PORT_InitTypeDef GPIOInitStruct;
  
  RST_CLK_PCLKcmd (RST_CLK_PCLK_PORTC, ENABLE);
  
  PORT_StructInit(&GPIOInitStruct);
  
  GPIOInitStruct.PORT_Pin        = LED_ALL;
  GPIOInitStruct.PORT_MODE       = PORT_MODE_DIGITAL;
  GPIOInitStruct.PORT_OE         = PORT_OE_OUT;
  GPIOInitStruct.PORT_SPEED      = PORT_SPEED_SLOW;
  
  PORT_Init(MDR_PORTC, &GPIOInitStruct);
}

void vError(){
  PORT_SetBits(MDR_PORTC, VD4);  // LED0 ON
  while(1);
}


void Delay(int waitTicks){
  int i;
  for (i = 0; i < waitTicks; i++){
    __NOP();
  }
}


#define BKP_RTC_CS_IE_Msk                        ((uint32_t)0x00000038)
/**
  * @brief  BKP_RTC_ClearFlagStatus - Clears the specified RTC flag.
  * @param  BKP_RTC_FLAG: specifies the flag to clear.
  *   This parameter can be any combination of the following values:
  *     @arg BKP_RTC_FLAG_OWF : Overflow flag
  *     @arg BKP_RTC_FLAG_ALRF: Alarm flag
  *     @arg BKP_RTC_FLAG_SECF: Second flag
  * @retval None.
  */
void BKP_RTC_ClearFlagStatus(uint32_t BKP_RTC_FLAG){
  uint32_t tmpreg;
  
  /* Check the parameters */
  assert_param(IS_BKP_RTC_GET_FLAG(BKP_RTC_FLAG));
  
  tmpreg = MDR_BKP->RTC_CS & BKP_RTC_CS_IE_Msk;
  tmpreg |= BKP_RTC_FLAG;
  
  MDR_BKP->RTC_CS = tmpreg;
}

